import json

class ImageError(Exception):
  pass

class result(object):
  def __init__(self, api, image=False, **kwarg):
    self.api=api
    self.image=image
    
  @property
  def titles(self):
    title=[]
    for e in self.api["items"]:
      i=e["title"]
      title.append(i)
    return title
    
  @property
  def urls(self):
    url=[]
    for e in self.api["items"]:
      i=e["link"]
      url.append(i)
    return url
    
  @property
  def display_urls(self):
    url=[]
    for e in self.api["items"]:
      i=e["displayLink"]
      url.append(i)
    return url
    
  @property
  def html_titles(self):
    title=[]
    for i in self.api["items"]:
      e=i["htmlTitle"]
      title.append(e)
    return title
    
  @property
  def snippets(self):
    snippet=[]
    for i in self.api["items"]:
      e=i["snippet"]
      snippet.append(e)
    return snippet
    
  @property
  def image_url(self):
    if self.image == True:
      image=[]
      for i in self.api["items"]:
        e=i["pagemap"]["cse_thumbnail"][0]["src"]
        image.append(e)
      return image
    else:
      raise ImageError("Image setting is False")


import requests
import aiohttp
import json
from .object import result

url="https://www.googleapis.com/customsearch/v1"

class ApiError(Exception):
  pass

class Engine_IdError(Exception):
  pass

class TokenError(Exception):
  pass

class custom_search(object): 
  def __init__(self, token=None, engine_id=None, image=False):
    if token is None:
      raise TokenError("token is invaid")
    if engine_id is None:
      raise Engine_IdError("engine_id is invaid")
    else: 
      self.token=token
      self.engine_id=engine_id
      self.image=image

  def search(self, keyword=None):
    if keyword is None:
      raise KeywordError("keyword is None")
    else:
      params={
        "key": self.token,
        "cx": self.engine_id,
        "q": keyword
      }
      res=requests.get(url,params=params)
      api=res.json()
      return result(api)
      
  async def search_async(self, keyword=None):
    if keyword is None:
      raise KeywordError("keyword is None")
    else:
      params={
        "key": self.token,
        "cx": self.engine_id,
        "q": keyword
      }
      async with aiohttp.ClientSession() as session:
        async with session.get(url, params=params) as res:
          api=await res.json()
          if api["error"]["code"] == 429:
            raise ApiError(api["error"]["message"])
          elif api["error"]["code"] == 400:
            raise ApiError(api["error"]["message"])
          return result(api, self.image)